<?php

namespace App\Http\Controllers\Admin;

use App\General\GeneralSettingsClass;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use App\Models\User;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\URL;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Hash;
use Exception;
use Illuminate\Support\Facades\Session;
use App\Http\Controllers\Admin\EmailController;

class AuthController extends Controller
{
    public function login(Request $request)
    {
        $credentials = $request->validate([
            'email' => 'required|email',
            'password' => 'required',
        ]);

        if (Auth::attempt($credentials)) {
            $request->session()->regenerate();
                $arr = DB::table('role_permissions')->where('role_id',Auth::user()->role_id)->get();
                    $permission = [];
                    $key = 0;
                    foreach($arr as $a)
                    {
                        $permission[$key] = $a->permission;
                        $key++;
                    }
                    
                $request->session()->put('permissions',$permission);

            return redirect()->intended('/admin');
        }

        return back()->withErrors([
            'email' => 'The provided credentials do not match our records.',
        ]);
    }

    public function logout()
    {
        Auth::logout();
        return redirect('/admin/login');
    }

    public function register(Request $request)
    {
        // Validation logic here...

        $user = User::create([
            'name' => $request->input('name'),
            'email' => $request->input('email'),
            'password' => Hash::make($request->input('password')),
        ]);

        Auth::login($user);

        return redirect('/dashboard');
    }
    public function sendResetLinkEmail(Request $request)
    {
        try {
            $request->validate([
                'email' => 'required|email',
            ]);
            $user = User::where('email', $request->email)->exists();
            $userDetail = User::where('email', $request->email)->first();
            if ($user && $userDetail->role != 5) {
                $logo = GeneralSettingsClass::getRequiredValue('_header_logo');
                // Delete previous password reset request if exists for this email
                DB::table('password_resets')->where('email', $request->email)->delete();

                $user_details = User::where('email', $request->email)->first();
                $token = Str::random(60);
                $expiration = now()->addMinutes(env('PASSWORD_LINK_TIME'));

                DB::table('password_resets')->insert([
                    'email' => $request->email,
                    'token' => $token,
                    'created_at' => now(),
                    'expires_at' => $expiration,
                ]);
                $resetUrl = URL::signedRoute('Admin.Auth.ResetPassword', ['token' => $token]);
                $params = [
                    'subject' => 'Reset Password Email | Yazi',
                    'to' => $user_details->email,
                    'blade' => 'reset-password',
                    'data' => [
                        'name' => $user_details->name,
                        'url' => $resetUrl,
                        'img_url' => url('/'),
                        'logo' => $logo,
                    ],
                ];

                $emailController = new EmailController();
                $emailSent = $emailController->SendInstantEmail($params);
                if ($emailSent) {
                    return response()->json([
                        'status' => 200,
                        'message' => 'An Reset Password Email has been sent to your Email.',
                    ]);
                } else {
                    return response()->json([
                        'status' => 500,
                        'message' => 'SMTP is not Working, Try Later!',
                    ]);
                }
            } else {
                return response()->json([
                    'status' => 404,
                    'message' => 'Account with the entered email does not exsit.',
                ]);
            }
        } catch (Exception $ex) {
            return response()->json([
                'status' => 500,
                'message' => 'An Error Occured, Try Again!',
                'error' => $ex,
            ]);
        }
    }

    public function updatePassword(Request $request)
    {
        $request->validate([
            'password' => 'required | confirmed | min:8',
            'token' => 'required'
        ]);

        $reset = DB::table('password_resets')
            ->where('token', $request->token)
            ->where('expires_at', '>', now())
            ->first();

        if (!$reset) {
            return response()->json([
                'status' => 404,
                'message' => 'Invalid or Expired Reset Password Link!',
            ]);
        }

        // Get the user's email address from the password_resets table
        $email = $reset->email;

        // Update the user's password
        $user = User::where('email', $email)->first();
        $user->password = Hash::make($request->password);
        $user->save();

        // Delete the token from the password_resets table
        DB::table('password_resets')->where('email', $email)->delete();
        $logo = GeneralSettingsClass::getRequiredValue('_header_logo');

        $params = [
            'subject' => 'Password Reset Successfully | Yazi',
            'to' => $user->email,
            'blade' => 'update-password',
            'data' => [
                'name' => $user->name,
                'url' => route('Admin.Login'),
                'email' => $email,
                'password' => $request->password,
                'img_url' => url('/'),
                'logo' => $logo,
            ],
        ];

        $emailController = new EmailController();
        $emailSent = $emailController->SendInstantEmail($params);
        if ($emailSent) {
            return response()->json([
                'status' => 200,
                'message' => 'Password Reset Confirmation Email has been sent to your Email.',
            ]);
        } else {
            return response()->json([
                'status' => 200,
                'message' => 'Password Updated, SMTP ERROR occured while sending email!',
            ]);
        }
    }
}
