<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\User;
use App\Models\AreaGroup;
use App\Models\Brand;
use App\Models\Product;
use App\Models\Collection;
use App\Models\order;
use App\Models\customer;
use App\Models\Invoices;
use App\Models\LeastSellingProduct;
use App\Models\productMeta;
use App\Models\SellingProduct;
use DB;
use App\Models\order_meta;

class DashboardController extends Controller
{
    public function index()
    {
        $total_users = User::count();
        $total_products = Product::where('active', 1)->count();
        $total_brands = Brand::where('active', 1)->count();
        $total_areagroups = AreaGroup::count();
        $total_collections = Collection::where('active', 1)->count();
        $all_orders = order::count();
        $comp_orders = order::where('status', 'completed')->count();
        $pend_orders = order::where('status', 'pending')->count();
        $canc_orders = order::where('status', 'cancelled')->count();
        $inpro_orders = order::where('status', 'inProcess')->count();
        $online_orders = order::where('channel', 'Online Store')->count();
        $customers = User::where('role_id', 5)->where('active', 1)
            ->orderBy('created_at', 'desc')
            ->take(7)
            ->get();
        $invoice_orders = Invoices::where('active', 1)->count();
        $orders = order::latest()->take(10)->get();

        $topSellingProducts = Product::select(
            'products.id',
            'products.title',
            'products.featured_image',
            'products.permalink',
            \DB::raw('COUNT(order_metas.id) as order_count'),
            \DB::raw('SUM(order_metas.quantity) as total_quantity')
        )
            ->leftJoin('order_metas', 'products.id', '=', 'order_metas.product_id')
            ->groupBy('products.id', 'products.title', 'products.featured_image', 'products.permalink')
            ->orderBy('order_count', 'DESC')
            ->take(7)
            ->get();

        $leastSellingProducts = Product::select(
            'products.id',
            'products.title',
            'products.featured_image',
            'products.permalink',
            \DB::raw('COUNT(order_metas.id) as order_count'),
            \DB::raw('SUM(order_metas.quantity) as total_quantity')
        )
            ->leftJoin('order_metas', 'products.id', '=', 'order_metas.product_id')
            ->groupBy('products.id', 'products.title', 'products.featured_image', 'products.permalink')
            ->orderBy('order_count', 'ASC')
            ->take(7)
            ->get();

        $sellingProducts = SellingProduct::with('product.productMeta')->get();
        $lessSellingProducts = LeastSellingProduct::get();
        // test
        $currentYear = date('Y');
        $comp_orders_data = Order::where('status', 'completed')
            ->selectRaw('SUM(total_amount) as total_amount')
            ->whereRaw('YEAR(created_at) = ?', [$currentYear])
            ->groupBy('created_at') // Group by the year
            ->get();
        $pos_orders_data = invoices::where('active', 1)
            ->selectRaw('SUM(grand_total) as grand_total')
            ->whereRaw('YEAR(created_at) = ?', [$currentYear])
            ->groupBy('created_at') // Group by the year
            ->get();

            $months = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];
            $monthData = [];
            $monthData2 = [];
            
            foreach ($months as $month) {
            $monthData[$month] = 0;
            $monthData2[$month] = 0;
            }
            
            foreach ($comp_orders_data as $data) {
            $monthData[$data->month] = floatval($data->total_amount);
            }
            foreach ($pos_orders_data as $data) {
            $monthData2[$data->month] = floatval($data->grand_total);
            }
            
        // test
        $orderData = Order::whereYear('created_at', $currentYear)
        ->selectRaw('MONTH(created_at) as month, SUM(total_amount) as total_sales')
        ->groupBy('month')
        ->get();

        
        

    // Retrieve data from 'invoices' table
    $invoiceData = Invoices::whereYear('created_at', $currentYear)
        ->selectRaw('MONTH(created_at) as month, SUM(grand_total) as total_sales')
        ->groupBy('month')
        ->get();
        
    $c_ =   count($orderData) >= count($invoiceData) ?  count($orderData) : count($invoiceData);
    
    // //  "month" => "9"
    // //     "total_sales" => "80"
    // for(var $i ; $i< $c_ ; $i++ )
    // {
    //     $invoiceData[$i]->month
    //     $orderData[$i]->month
    // }

    // Create and return the chart data
    $chartData = [
        'labels' => $orderData->pluck('month')->toArray(),
        'orderSales' => $orderData->pluck('total_sales')->toArray(),
        'posSales' => $invoiceData->pluck('total_sales')->toArray(),
    ];

        return view('Admin.dashboard', compact('total_users', 'total_products', 'total_brands', 'total_areagroups', 'total_collections', 'all_orders', 'comp_orders', 'pend_orders', 'canc_orders', 'inpro_orders', 'online_orders', 'customers', 'invoice_orders', 'orders', 'topSellingProducts', 'sellingProducts', 'leastSellingProducts', 'lessSellingProducts','months', 'monthData','monthData2','currentYear','chartData'));
    }


    public function updateTopSellingProducts()
    {
        $topSellingProducts = Product::select(
            'products.id',
            \DB::raw('COUNT(order_metas.id) as order_count'),
            \DB::raw('SUM(order_metas.quantity) as total_quantity')
        )
            ->leftJoin('order_metas', 'products.id', '=', 'order_metas.product_id')
            ->groupBy('products.id')
            ->orderBy('order_count', 'DESC')
            ->take(7)
            ->get();

        SellingProduct::truncate();
        foreach ($topSellingProducts as $product) {
            SellingProduct::create([
                'product_id' => $product->id,
                'count' => $product->order_count,
                'total_quantity' => $product->total_quantity,
            ]);
        }

        return 'Top selling products updated successfully.';
    }

    public function updateLeastSellingProducts()
    {
        $leastSellingProducts = Product::select('products.id', 'products.title', 'products.featured_image', 'products.permalink', \DB::raw('COUNT(order_metas.id) as order_count'))
            ->leftJoin('order_metas', 'products.id', '=', 'order_metas.product_id')
            ->groupBy('products.id', 'products.title', 'products.featured_image', 'products.permalink')
            ->orderBy('order_count', 'ASC')
            ->take(7)
            ->get();
        LeastSellingProduct::truncate();
        foreach ($leastSellingProducts as $product) {
            LeastSellingProduct::create([
                'product_id' => $product->id,
                'title' => $product->title,
                'featured_image' => $product->featured_image,
                'permalink' => $product->permalink,
                'quantity' => $product->total_quantity,
                'order_count' => $product->order_count
            ]);
        }

        return 'Least selling products updated successfully.';
    }
}
