<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\Product;
use App\Models\ProductMeta;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;

class InventoryController extends Controller
{
    public function index()
    {
        $activeStatuses = [0, 1, 2];
        $products = Product::whereIn('active', $activeStatuses)->with('productMeta')->paginate(30);
        return view('Admin.Products.inventory', compact('products'));
    }

    public function update(Request $request)
    {
        // dd($request);
        $stocks = $request->input('_stock', []);
        $prices = $request->input('_price', []);
        $status = $request->input('active', []);


        foreach ($status as $rowId => $row) {
            $product = Product::find($rowId);
            if ($product) {
                $product->update([
                    'active' => $row,
                ]);
                $product_meta = ProductMeta::where('product_id', $product->id)->get();
                if ($product_meta) {
                    $meta_price = $product_meta->where('option', '_price')->first();
                    $meta_price->value = $prices[$rowId];
                    $meta_price->update();

                    $meta_stock = $product_meta->where('option', '_stock')->first();
                    $meta_stock->value = $stocks[$rowId];
                    $meta_stock->update();
                }
            }
        }
        return redirect()->back()->with(['success' => "Product's Inventory is updated Successfully"]);
    }

    public function search(Request $request)
    {
        $activeStatuses = [0, 1, 2];
        $query = Product::query()->leftjoin('products_meta', 'products.id', '=', 'products_meta.product_id')->select('products.*');
        if ($request->value) {
            $query->where('products.title', 'LIKE', '%' . $request->value . '%')
                ->orWhere('products.sku', 'LIKE', '%' . $request->value . '%')
                ->orWhere('products_meta.value', 'LIKE', '%' . $request->value . '%');
        }
        $products = $query->whereIn('active', $activeStatuses)->distinct()->with('productMeta')->paginate(30);

        if (!$products->isEmpty()) {
            return view('Admin.Products.ajax.filtered_inventory', compact('products'))->render();
        }
    }

    public function import_data(Request $request)
    {
        if ($request->hasFile('file')) {
            $logged_in_user = Auth::user();
            $file = $request->file('file');
            # Read and parse the CSV file
            $csvData = array_map('str_getcsv', file($file));
            # Define the expected columns
            $expectedColumns = [
                "PRODUCT ID", "PRODUCT", "SKU", "STOCK", "PRICE"
            ];
            # Remove the header row
            array_shift($csvData);
            foreach ($csvData as $row) {
                DB::table('products_meta')->where('product_id', $row[0] ?? 0)->where('option', '_stock')->update(['value' => $row[3] ?? null]);
                DB::table('products_meta')->where('product_id', $row[0] ?? 0)->where('option', '_price')->update(['value' => $row[4] ?? null]);
            }


            return redirect()->back()->with('success', 'CSV data imported successfully11.');
        } else {
            return redirect()->back()->with('success', 'CSV File not found');
        }
    }
}
