<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Http\Controllers\Admin\EmailController;
use App\Models\order;
use App\Models\order_meta;
use App\Models\User;
use App\Models\GeneralSettings;
use App\General\GeneralSettingsClass;
use App\Http\Controllers\Payment\Traits\RefundPaymentTrait;
use App\Models\ReturnOrder;
use Carbon\Carbon;
use Illuminate\Contracts\Session\Session;
use Illuminate\Http\Request;

class POSController extends Controller
{
    use RefundPaymentTrait;
    public function index(Request $request)
    {
        session()->forget([
            'defaultFilterDays', 'totalOrders', 'completedOrders', 'pendingOrders',
            'inProcessOrders', 'cancelledOrders',
        ]);
        $now = Carbon::now();
        $sevenDaysAgo = $now->copy()->subDays(7);
        $originalQuery = order::whereBetween('created_at', [$sevenDaysAgo, $now])->where('admin_delete', 0)->where('status', '!=', 'refund')->orderBy('id', 'desc')->with('orderMeta', 'user');
        $perPage = 15;
        $orders = $originalQuery->paginate($perPage);

        $completedQuery = clone $originalQuery;
        $pendingQuery = clone $originalQuery;
        $inProcessQuery = clone $originalQuery;
        $cancelledQuery = clone $originalQuery;

        $totalOrders = $originalQuery->count();
        $completedOrders = $completedQuery->where('status', 'completed')->count();
        $pendingOrders = $pendingQuery->where('status', 'pending')->count();
        $inProcessOrders = $inProcessQuery->where('status', 'inProcess')->count();
        $cancelledOrders = $cancelledQuery->where('status', 'cancelled')->count();

        session([
            'defaultFilterDays' => 7,
            'totalOrders' => $totalOrders,
            'completedOrders' => $completedOrders,
            'pendingOrders' => $pendingOrders,
            'inProcessOrders' => $inProcessOrders,
            'cancelledOrders' => $cancelledOrders,
        ]);

        return view('Admin.Orders.index', compact('orders', 'totalOrders', 'completedOrders', 'pendingOrders', 'cancelledOrders', 'inProcessOrders'));
    }

    public function filter_order(Request $request)
    {
        if ($request->ajax()) {
            $now = Carbon::now();
            $ajaxFilterDays = $request->filterDays;
            $page = $request->page;
            $pageType = $request->pageType;

            $ajaxDaysAgo = $now->copy()->subDays($ajaxFilterDays);
            if ($pageType == 'Returned') {
                if ($ajaxFilterDays == null) {
                    $originalQuery = order::where('status', 'refund')->where('admin_delete', 0)->orderBy('id', 'desc')->with('orderMeta', 'user', 'returnedOrder');
                } else {
                    $originalQuery = order::where('status', 'refund')->whereBetween('created_at', [$ajaxDaysAgo, $now])->where('admin_delete', 0)->orderBy('id', 'desc')->with('orderMeta', 'user');
                }
            } else {
                if ($ajaxFilterDays == null) {
                    $originalQuery = order::where('admin_delete', 0)->where('status', '!=', 'refund')->orderBy('id', 'desc')->with('orderMeta', 'user');
                } else {
                    $originalQuery = order::whereBetween('created_at', [$ajaxDaysAgo, $now])->where('admin_delete', 0)->where('status', '!=', 'refund')->orderBy('id', 'desc')->with('orderMeta', 'user');
                }
            }
            $perPage = 15;
            $orders = $originalQuery->paginate($perPage);
            $paginationHtml = $orders->links('pagination::bootstrap-4')->toHtml();

            $saveFilterDays = $ajaxFilterDays;

            if (session()->get('defaultFilterDays') != $ajaxFilterDays) {
                $completedQuery = clone $originalQuery;
                $pendingQuery = clone $originalQuery;
                $inProcessQuery = clone $originalQuery;
                $cancelledQuery = clone $originalQuery;

                $totalOrders = $originalQuery->count();
                $completedOrders = $completedQuery->where('status', 'completed')->count();
                $pendingOrders = $pendingQuery->where('status', 'pending')->count();
                $inProcessOrders = $inProcessQuery->where('status', 'inProcess')->count();
                $cancelledOrders = $cancelledQuery->where('status', 'cancelled')->count();

                session([
                    'defaultFilterDays' => $ajaxFilterDays,
                    'totalOrders' => $totalOrders,
                    'completedOrders' => $completedOrders,
                    'pendingOrders' => $pendingOrders,
                    'inProcessOrders' => $inProcessOrders,
                    'cancelledOrders' => $cancelledOrders,
                ]);
            } else {
                $totalOrders = session('totalOrders', null);
                $completedOrders = session('completedOrders', null);
                $pendingOrders = session('pendingOrders', null);
                $inProcessOrders = session('inProcessOrders', null);
                $cancelledOrders = session('cancelledOrders', null);
            }
            
            if ($pageType == 'Returned') {
                $orders = view('Admin.Ajax.returnedFilteredOrders', compact('orders'))->render();
                $counts = view('Admin.Ajax.returnedFilteredOrderCount', compact('ajaxFilterDays', 'totalOrders', 'pendingOrders', 'completedOrders', 'inProcessOrders', 'cancelledOrders'))->render();
            } else {
                $orders = view('Admin.Ajax.filteredOrders', compact('orders'))->render();
                $counts = view('Admin.Ajax.filteredOrderCount', compact('ajaxFilterDays', 'totalOrders', 'pendingOrders', 'completedOrders', 'inProcessOrders', 'cancelledOrders'))->render();
            }
            return response()->json([
                'orders' => $orders,
                'counts' => $counts,
                'pagination' => $paginationHtml,
            ]);
        }
    }
    public function change_order_status(Request $request)
    {
        $orderId = $request->input('orderId');
        $status = $request->input('status');

        $order = order::where('id', $orderId)->update([
            'status' => $status,
        ]);

        if ($status == 'completed' || $status == "cancelled") {
            $capitalizedStatus = ucwords($status);
            $orderdetail = order::where('id', $orderId)->with('orderAddressDetail')->first();
            $orderMeta = order_meta::where('order_id', $orderdetail->id)->with('product')->get();

            $this->sendStatusChangeEmail($capitalizedStatus, $orderdetail, $orderMeta, $status);
        }
        if ($order) {
            return response()->json(['status' => true, 'message' => 'Order Status Updated Successfully']);
        } else {
            return response()->json(['status' => false, 'message' => 'Failed to Update Order Status']);
        }
    }

    public function detail(Request $request)
    {
        $order = order::where('id', $request->id)->with('orderMetas.product')->first();
        $user = null;
        $orderCount = 0;
        if ($order->user_id) {
            $user = User::where('id', $order->user_id)->orWhere('email', $order->email)->with('userMeta')->first();
            $orderCount = order::where('user_id', $user->id)->orWhere('email', $order->email)->count();
        }
        return view('Admin.Orders.details', compact('order', 'user', 'orderCount'))->render();
    }
    public function delete_order(Request $request)
    {
        $id = $request->id;
        $order = order::find($id);
        if ($order) {
            $order->admin_delete = 1;
            $order->update();
            return response()->json(['status' => 'success', 'message' => 'Order has been deleted.']);
        } else {
            return response()->json(['status' => 'error', 'message' => 'Order not found.']);
        }
    }

    public function bulk_delete(Request $request)
    {
        $ids = $request->input('ids');
        order::whereIn('id', $ids)->update([
            'admin_delete' => 1
        ]);
        return response()->json(['message' => 'Selected records deleted successfully']);
    }
    
    public function cancelled_orders_list()
    {
        $orders = order::where('status', 'cancelled')->paginate(30);
        return view('Admin.Orders.cancelled', compact('orders'));
    }
    
    public function return_order_list()
    {
        session()->forget([
            'defaultFilterDays', 'totalOrders', 'completedOrders', 'pendingOrders',
            'inProcessOrders', 'cancelledOrders',
        ]);
        $now = Carbon::now();
        $sevenDaysAgo = $now->copy()->subDays(7);
        $originalQuery = order::where('status', 'refund')->whereBetween('created_at', [$sevenDaysAgo, $now])->where('admin_delete', 0)->orderBy('id', 'desc')->with('orderMeta', 'user', 'returnedOrder');
        $perPage = 15;
        $orders = $originalQuery->paginate($perPage);

        $completedQuery = clone $originalQuery;
        $pendingQuery = clone $originalQuery;
        $inProcessQuery = clone $originalQuery;
        $cancelledQuery = clone $originalQuery;

        $totalOrders = $originalQuery->count();
        $completedOrders = $completedQuery->where('status', 'completed')->count();
        $pendingOrders = $pendingQuery->where('status', 'pending')->count();
        $inProcessOrders = $inProcessQuery->where('status', 'inProcess')->count();
        $cancelledOrders = $cancelledQuery->where('status', 'cancelled')->count();

        session([
            'defaultFilterDays' => 7,
            'totalOrders' => $totalOrders,
            'completedOrders' => $completedOrders,
            'pendingOrders' => $pendingOrders,
            'inProcessOrders' => $inProcessOrders,
            'cancelledOrders' => $cancelledOrders,
        ]);

        return view('Admin.OrderReturn.returned', compact('orders', 'totalOrders', 'completedOrders', 'pendingOrders', 'cancelledOrders', 'inProcessOrders'));
    }
    
    public function return_detail(Request $request)
    {
        $order = ReturnOrder::where('id', $request->id)->with('returnMeta.product', 'order')->first();
        $user = null;
        $orderCount = 0;
        if ($order->user_id) {
            $user = User::where('id', $order->user_id)->orwhere('email', $order->user_email)->with('userMeta')->first();
            $orderCount = ReturnOrder::where('user_id', $user->id)->orwhere('user_email', $order->user_email)->count();
        }
        return view('Admin.OrderReturn.return-detail', compact('order', 'user', 'orderCount'))->render();
    }

    public function delete_returned_order(Request $request)
    {
        $id = $request->id;
        $order = ReturnOrder::find($id);
        if ($order) {
            $order->admin_delete = 1;
            $order->update();
            return response()->json(['status' => 'success', 'message' => 'Order has been deleted.']);
        } else {
            return response()->json(['status' => 'error', 'message' => 'Order not found.']);
        }
    }

    public function bulk_delete_returned_order(Request $request)
    {
        $ids = $request->input('ids');
        ReturnOrder::whereIn('id', $ids)->update([
            'admin_delete' => 1
        ]);
        return response()->json(['message' => 'Selected records deleted successfully']);
    }

    public function sendStatusChangeEmail($capitalizedStatus, $orderdetail, $orderMeta, $status)
    {
        $data = GeneralSettings::where('option', '_header_logo')->orWhere('option', '_b_email')->orWhere('option', '_instagram_link')->orWhere('option', '_facebook_link')->orWhere('option', '_twitter_link')->orWhere('option', '_youtube_link')->get()->toArray();

        $admin_email_index = array_search('_b_email', array_column($data, 'option'));
        $header_logo_index = array_search('_header_logo', array_column($data, 'option'));
        $insta_link_index = array_search('_instagram_link', array_column($data, 'option'));
        $fb_link_index = array_search('_facebook_link', array_column($data, 'option'));
        $twitter_link_index = array_search('_twitter_link', array_column($data, 'option'));
        $youtube_link_index = array_search('_youtube_link', array_column($data, 'option'));

        $admin_email = $data[$admin_email_index]['value'];
        $logo = $data[$header_logo_index]['value'];
        $instagram_link = $data[$insta_link_index]['value'];
        $facebook_link = $data[$fb_link_index]['value'];
        $twitter_link = $data[$twitter_link_index]['value'];
        $youtube_link = $data[$youtube_link_index]['value'];

        $EMAIL = null;
        
        if($orderdetail && $orderdetail->orderAddressDetail && $orderdetail->orderAddressDetail->email)
        {
            
            $EMAIL = $orderdetail->orderAddressDetail->email;
            
        }
        elseif($orderdetail && $orderdetail->email)
        {
            $EMAIL = $orderdetail->email;
        }

        $params = [
            'subject' => 'Order ' . $capitalizedStatus . ' | ' . config('app.website_name'),
            'to' => $EMAIL,
            'blade' => 'order',
            'data' => [
                'name' => $orderdetail->f_name . " " . $orderdetail->l_name,
                'status' => $status,
                'orderdetail' => $orderdetail,
                'orderMeta' => $orderMeta,
                'url' => url('/'),
                'logo' => $logo,
                'instagram_link' => $instagram_link,
                'facebook_link' => $facebook_link,
                'twitter_link' => $twitter_link,
                'youtube_link' => $youtube_link,
                'admin_email' => $admin_email,
            ],
        ];
        $emailController = new EmailController();
        $emailSent = $emailController->SendInstantEmail($params);
    }
}
