<?php

namespace App\Http\Controllers\Admin;

use App\General\GeneralSettingsClass;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\User;
use App\Models\UserMeta;
use App\Models\GeneralSettings;
use DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Validation\Rule;
use App\Http\Controllers\Admin\EmailController;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\URL;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Auth;

class UserController extends Controller
{
    public function index()
    {
        $users = User::where('role_id','!=',5)->get();
        $roles = DB::table('roles')->get();
        return view('Admin.Users.index', compact('users','roles'));
    }

    public function edit(User $User)
    {
        $User->load('userMeta');
        $roles = DB::table('roles')->get();
        return view('Admin.Users.edit', compact('User','roles'));
    }

    public function update(Request $request, $id)
    {
        $sanitizedData = $request->validate([
            'name' => 'required|string|max:255',
            'email' => [
                'email',
                Rule::unique('users')->ignore($id),
            ],
            'phone' => 'required|string|max:15',
            'role_id' => 'required|integer',
        ]);

        $sanitizedData['profile_image'] = '/user.jpg';

        if (isset($request->created_by)) {
            $sanitizedData['created_by'] = $request->created_by;
            $sanitizedData['email_verified_at'] = now();
            $sanitizedData['active'] = 1;
        }

        $user = User::find($request->user_id);
        if (!$user) {
            return response()->json([
                'status' => 404,
                'message' => 'User not found.',
            ]);
        }
          $user->update($sanitizedData);
          $metaDescriptions = $request->only([
            '_b_name',
            '_b_email',
            '_b_number',
            '_b_street_address',
            '_b_area_address'
        ]);

        $userMeta = [];
        foreach ($metaDescriptions as $option => $value) {
            if ($value !== null) {
                $userMeta[] = [
                    'user_id' => $user->id,
                    'option' => $option,
                    'value' => $value,
                ];
            }
        }

    # Delete previous product meta descriptions
        DB::table('user_metas')->where('user_id', $user->id)->delete();
        DB::table('user_metas')->insert($userMeta);

        if ($user) {
            $emailController = new EmailController();

            if (isset($request->confirmation_email_sending)) {
                $params = [
                    'subject' => 'Account Login Details | Yazi',
                    'to' => $request->email,
                    'blade' => 'account-login-details', // the email blade name
                    'data' => [
                        'name' => $request->name,
                        'email' => $request->email,
                        'password' => $request->password,
                        'url' => route('Admin.Login')
                    ],
                ];

                $emailSent = $emailController->SendInstantEmail($params);

                if ($emailSent) {
                    return response()->json([
                        'status' => 200,
                        'message' => 'Account Updated successfully, An Email has been sent to user email for account information.',
                    ]);
                } else {
                    return response()->json([
                        'status' => 500,
                        'message' => 'Account Upated successfully, An Error Occurred while sending the email.',
                    ]);
                }
            }


            return response()->json([
                        'status' => 200,
                        'message' => 'Account Updated successfully.',
                    ]);
        } else {
            return response()->json([
                'status' => 500,
                'message' => 'An Error Occurred, During Updated user account.',
            ]);
        }
    }

    public function customers()
    {
        $users = User::all();
        $customers = User::where('role_id',5)->get();
        $roles = DB::table('roles')->get();
        return view('Admin.Users.customer', compact('users','customers','roles'));
    }

    public function create()
    {
        $roles = DB::table('roles')->get();
        return view('Admin.Users.create', compact('roles'));
    }

    public function show(User $user)
    {
        $users = User::all();
        $roles = DB::table('roles')->get();
        $UserMeta = UserMeta::where('user_id', $user->id)->get();
        return response()->json([
            'status' => 200,
            'user_details' => $user,
            'roles' => $roles,
            'users' => $users,
            'user_meta' => $UserMeta
        ]);
    }

    public function store(Request $request)
    {
        $sanitizedData = $request->validate([
            'name' => 'required|string|max:255',
            'email' => [
                'required',
                'email',
                Rule::unique('users'),
            ],
            'password' => 'required|min:8',
            'phone' => 'required|string|max:15',
            'role_id' => 'required|integer',
        ]);

        $sanitizedData['profile_image'] = '/user.jpg';

        if (isset($request->created_by)) {
            $sanitizedData['created_by'] = $request->created_by;
            $sanitizedData['email_verified_at'] = now();
            $sanitizedData['active'] = 1;
        }

        $sanitizedData['password'] = Hash::make($sanitizedData['password']);

        $user = User::create($sanitizedData);
        $metaDescriptions = $request->only([
            '_b_name',
            '_b_email',
            '_b_number',
            '_b_street_address',
            '_b_area_address'
        ]);

        $userMeta = [];
        foreach ($metaDescriptions as $option => $value) {
            if ($value !== null) {
                $userMeta[] = [
                    'user_id' => $user->id,
                    'option' => $option,
                    'value' => $value,
                ];
            }
        }

        # Insert user meta descriptions in bulk
        DB::table('user_metas')->insert($userMeta);
        $customers = User::where('role_id', 5)->where('active' ,1)->get();

        $data = GeneralSettings::where('option', '_header_logo')->orWhere('option', '_b_email')->orWhere('option', '_instagram_link')->orWhere('option', '_facebook_link')->orWhere('option', '_twitter_link')->orWhere('option', '_youtube_link')->get()->toArray();

        $admin_email_index = array_search('_b_email', array_column($data, 'option'));
        $header_logo_index = array_search('_header_logo', array_column($data, 'option'));
        $insta_link_index = array_search('_instagram_link', array_column($data, 'option'));
        $fb_link_index = array_search('_facebook_link', array_column($data, 'option'));
        $twitter_link_index = array_search('_twitter_link', array_column($data, 'option'));
        $youtube_link_index = array_search('_youtube_link', array_column($data, 'option'));

        $admin_email = $data[$admin_email_index]['value'];
        $logo = $data[$header_logo_index]['value'];
        $instagram_link = $data[$insta_link_index]['value'];
        $facebook_link = $data[$fb_link_index]['value'];
        $twitter_link = $data[$twitter_link_index]['value'];
        $youtube_link = $data[$youtube_link_index]['value'];

        if ($user) {
            $route = route('Admin.Login');
            if($user->role_id == 5){
                $route = url('/') . '/sign-in';
            }
            else{
                $route = route('Admin.Login');
            }
            $emailController = new EmailController();

            if (isset($request->confirmation_email_sending)) {
                $params = [
                    'subject' => 'Account Login Details | Yazi',
                    'to' => $request->email,
                    'blade' => 'account-login-details',
                    'data' => [
                        'name' => $request->name,
                        'email' => $request->email,
                        'password' => $request->password,
                        'url' => $route,
                        'image_url' => url('/'),
                        'logo' => $logo,
                        'instagram_link' => $instagram_link,
                        'facebook_link' => $facebook_link,
                        'twitter_link' => $twitter_link,
                        'youtube_link' => $youtube_link,
                        'admin_email' => $admin_email,
                    ],
                ];

                $emailSent = $emailController->SendInstantEmail($params);

                if ($emailSent) {
                    return response()->json([
                        'status' => 200,
                        'customers' => $customers,
                        'message' => 'Account created successfully, An Email has been sent to user email for account information.',
                    ]);
                } else {
                    return response()->json([
                        'status' => 500,
                        'customers' => $customers,
                        'message' => 'Account created successfully, An Error Occurred while sending the email.',
                    ]);
                }
            }


            return response()->json([
                        'status' => 200,
                        'customers' => $customers,
                        'message' => 'Account created successfully.',
                    ]);
        } else {
            return response()->json([
                'status' => 500,
                'message' => 'An Error Occurred, During creating user account.',
            ]);
        }
    }

    public function destroy(User $user)
    {
        $redirectRoute = ($user->role_id == 5) ? 'Customer.List' : 'users.index';

        $user->delete();

        return redirect()->route($redirectRoute);
    }

    public function updateStatus(Request $request, User $user)
    {
        $request->validate([
            'active' => 'required',
        ]);

        $user->active = $request->active;
        $user->save();

        return response()->json([
            'status' => 200,
            'message' => 'User status updated successfully.',
        ]);
    }

    public function updatePassword(Request $request)
    {
        $request->validate([
            'old_password' => 'required',
            'password' => 'required|min:8|confirmed',
        ]);

        $user = Auth::user();
        $oldPassword = $request->old_password;
        $newPassword = $request->password;

        if (Hash::check($oldPassword, $user->password)) {
            $user->password = Hash::make($newPassword);
            $user->save();

            return response()->json([
                'status' => 'success',
                'message' => 'Password updated successfully.',
            ]);
        } else {
            return response()->json([
                'status' => 'error',
                'message' => 'Invalid old password.',
            ], 422);
        }
    }

    public function updateProfileImage(Request $request)
    {
        $request->validate([
            'profile_image' => 'image|mimes:jpeg,png,jpg,gif|max:2048',
        ]);

        if ($request->hasFile('profile_image')) {
            $user = Auth::user();
            $image = $request->file('profile_image');
            $imageName = time() . '.' . $image->getClientOriginalExtension();
            $image->move(public_path('AdminAssets/Users/'), $imageName);

            $user->profile_image = '/AdminAssets/Users/' . $imageName;
            $user->save();

            // Update user session with the new profile image
            $request->session()->put('user.profile_image', $user->profile_image);

            return response()->json([
                'status' => 'success',
                'image_link' => url('/') . $user->profile_image,
                'message' => 'Profile image updated successfully. Refresh Page to implement Changes',
            ]);
        }

        return response()->json([
            'status' => 'error',
            'message' => 'No image file provided.',
        ], 422);
    }
}
